@@ -29,9 +29,13 @@ from zc.buildout import UserError
 from slapos.recipe.librecipe import wrap, JSON_SERIALISED_MAGIC_KEY
 import json
 from slapos import slap as slapmodule
+from slapos.slap import SoftwareProductCollection
 import slapos.recipe.librecipe.generic as librecipe
 import traceback
 
+import six
+
+SOFTWARE_PRODUCT_NAMESPACE = "product."
 DEFAULT_SOFTWARE_TYPE = 'RootSoftwareInstance'
 
 class Recipe(object):
@@ -108,10 +112,10 @@ class Recipe(object):
       raise UserError("'config' & 'sla' options are obsolete."
                       " Clean up your software release.")
     filter_kw = {k[4:]: v
-      for k, v in options.iteritems()
+      for k, v in six.iteritems(options)
       if k.startswith('sla-') and v}
     partition_parameter_kw = self._filterForStorage({k[7:]: v
-      for k, v in options.iteritems()
+      for k, v in six.iteritems(options)
       if k.startswith('config-')})
     slave = options.get('slave', 'false').lower() in \
       librecipe.GenericBaseRecipe.TRUE_VALUES
@@ -130,6 +134,19 @@ class Recipe(object):
       options['computer-id'],
       options['partition-id'],
     ).request
+
+    if software_url is not None and \
+      software_url.startswith(SOFTWARE_PRODUCT_NAMESPACE):
+    
+      product = SoftwareProductCollection(self.logger, slap)
+      
+      try:
+        software_url = product.__getattr__(
+          software_url[len(SOFTWARE_PRODUCT_NAMESPACE):])
+      except AttributeError as e:
+        self.logger.warning('Error on get software release : %s ' % e.message)
+        
+
     self._raise_request_exception = None
     self._raise_request_exception_formatted = None
     self.instance = None
@@ -145,27 +162,45 @@ class Recipe(object):
       # Note: SlapOS Master does not support it for slave instances
       if not slave:
         try:
-          options['instance-guid'] = self.instance.getInstanceGuid()
+          options['instance-guid'] = self.instance.getInstanceGuid() \
+              .encode('UTF-8')
           # XXX: deprecated, to be removed
-          options['instance_guid'] = self.instance.getInstanceGuid()
+          options['instance_guid'] = options['instance-guid']
           options['instance-state'] = self.instance.getState()
           options['instance-status'] = self.instance.getStatus()
         except (slapmodule.ResourceNotReady, AttributeError):
           # Backward compatibility. Old SlapOS master and core don't know this.
           self.logger.warning("Impossible to fetch instance GUID nor state.")
     except (slapmodule.NotFoundError, slapmodule.ServerError, slapmodule.ResourceNotReady) as exc:
+      self.logger.warning(
+        'Request for %(request_name)r with software release '
+        '%(software_release)r and software type %(software_type)r failed '
+        'with partition_parameter_kw=%(partition_parameter_kw)r, '
+        'filter_kw=%(filter_kw)r, shared=%(shared)r, state=%(state)r.', dict(
+          software_release=software_url,
+          software_type=software_type,
+          request_name=name,
+          partition_parameter_kw=partition_parameter_kw,
+          filter_kw=filter_kw,
+          shared=slave,
+          state=requested_state
+        )
+      )
       self._raise_request_exception = exc
       self._raise_request_exception_formatted = traceback.format_exc()
       return_parameter_dict = {}
 
     # Then try to get all the parameters. In case of problem, put empty string.
     for param in return_parameters:
-      options['connection-%s' % param] = ''
+      value = ''
       try:
-        options['connection-%s' % param] = return_parameter_dict[param]
+        value = return_parameter_dict[param]
       except KeyError:
         if self.failed is None:
           self.failed = param
+      if six.PY2 and isinstance(value, unicode):
+        value = value.encode('UTF-8')
+      options['connection-%s' % param] = value
 
   def _filterForStorage(self, partition_parameter_kw):
     return partition_parameter_kw
@@ -234,7 +269,7 @@ class RequestOptional(Recipe):
 
   update = install
 
-class Serialised(Recipe):
+class JSONCodec(object):
   def _filterForStorage(self, partition_parameter_kw):
     return wrap(partition_parameter_kw)
 
@@ -244,7 +279,17 @@ class Serialised(Recipe):
     except slapmodule.NotFoundError:
       return {}
 
+class RequestJSONEncoded(JSONCodec, Recipe):
+  """
+  Like Recipe, but serialised with JSONCodec.
+  """
+  pass
 
+class RequestOptionalJSONEncoded(JSONCodec, RequestOptional):
+  """
+  Like RequestOptional, but serialised with JSONCodec.
+  """
+  pass
 
 
 CONNECTION_PARAMETER_STRING = 'connection-'
@@ -267,12 +312,12 @@ class RequestEdge(Recipe):
 
       self.request_dict[country] = Recipe(buildout, name, local_options)
       # "Bubble" all connection parameters
-      for option, value in local_options.iteritems():
+      for option, value in six.iteritems(local_options):
         if option.startswith(CONNECTION_PARAMETER_STRING):
           self.options['%s-%s' % (option, country)] = value
 
   def install(self):
-    for country, request in self.request_dict.iteritems():
+    for country, request in six.iteritems(self.request_dict):
       request.install()
     return []
 
